/*
 *
 * CDinfo: get info about CD
 *
 *   Copyright (C) 2005  Pierre Ficheux
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <cdaudio.h>

#define CD_DEV "/dev/cdrom"

#define GET_ARTIST        1
#define GET_TITLE         2
#define GET_TRACKS        4
#define GET_GENRE         8

static char device_name[256];

main (int ac, char **av)
{
  char *cp;
  int fd_cd, disc_id, i, mask=0, r=0;
  struct disc_info disc_info;
  struct disc_data disc_data;

  strcpy (device_name, CD_DEV);

  while (--ac) {
    if ((cp = *++av) == NULL)
      break;
    if (*cp == '-' && *++cp) {

      switch(*cp) {
      case 'd' :
	strcpy (device_name, *++av); break;

      case 'a' :
	mask |= GET_ARTIST; break;

      case 'g' :
	mask |= GET_GENRE; break;

      case 't' :
	mask |= GET_TITLE; break;

      case 'T' :
	mask |= GET_TRACKS; break;

      default: 
	fprintf (stderr, "Usage: %s [-a] [-d device_name] [-g] [-t] [-T]\n");
	exit (1);
      }
    }
  }


  /* Open CD */
  if ((fd_cd = cd_init_device (device_name)) < 0) {
    perror (device_name);
    exit (1);
  }

  if (cd_stat (fd_cd, &disc_info) < 0) {
    perror ("cd_stat");
    exit (1);
  }

  /* Get ID + query CDDB */
  disc_id = cddb_direct_discid (disc_info);

  if (cddb_read_disc_data (fd_cd, &disc_data) < 0) 
    fprintf (stderr, "Can't access CDDB for %x!", disc_id);
  else {
    if (cddb_write_disc_data (fd_cd, disc_data) < 0)
      perror ("cddb_write_disc_data");

    if (mask & GET_ARTIST) {
      if (disc_data.data_artist[0] != 0)
	printf ("%s\n", disc_data.data_artist);
      else
	r = 2;
    }
    

    if (mask & GET_TITLE) {
      if (disc_data.data_title[0] != 0)
	printf ("%s\n", disc_data.data_title);
      else
	r = 2;
    }

    if (mask & GET_GENRE) {
      char *s = cddb_genre(disc_data.data_genre);
      if (s && *s) 
	printf ("%s\n", s);
      else
	r = 2;
    }

    if (mask & GET_TRACKS) {
      for (i = 0 ; i < disc_info.disc_total_tracks ; i++) {
	if (disc_data.data_track[i].track_name[0] != 0)
	  printf ("%02d_%s\n", i+1, disc_data.data_track[i].track_name);
	else
	  r = 2;
      }
    }
  }

  /* Close CD */
  cd_close (fd_cd);

  exit (r);
}
