#!/bin/sh
# 
# Smart old-style MP3 ripper
#
# (c)2005 Pierre Ficheux (pierre@ficheux.org)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

# Get CD info from FreeDB.org
CDINFO=cdinfo

# Default CD device
CDDEV=/dev/cdrom

# Default iPod mount point
IPODDIR=/mnt/ipod


# Usage function
usage () {
cat <<EOF
MP3 RIP+TAG utility, available options:
-d  set device (default is /dev/cdrom)
-h  this help
-t  don't rip, tag only existing files
EOF
}

# Confirm action
my_confirm ()
{
echo -n "$1 ? [y/n] "
read C
if [ "$C" != "y" -a "$C" != "Y" ]; then
    return 0
else
    return 1
fi
}

LIST=/tmp/myrip$$
TAGONLY=0
TOOLS="cdparanoia gogo mp3info ${CDINFO}"

# Command line options
while true
    do
     case "$1" in
      "-h") usage ; exit 0;;
      "-d") CDDEV=$2 ; shift;;
      "-t") TAGONLY=1;;
      "-"*) echo Invalid option \'$1\'
	    usage ; exit 1;;
      *) break;;
     esac
     shift
done

# Check tools
for i in $TOOLS
do
  type $i 1>/dev/null 2>&1
  if [ $? -ne 0 ]; then
      echo "\"$i\" is not found on your system, exiting."
      exit 1
  fi
done

# Check FreeDB.org
echo -n "Checking FreeDB.org..."

# Get artist + title + genre
ARTIST=`${CDINFO} -d ${CDDEV} -a`
TITLE=`${CDINFO} -d ${CDDEV} -t`
GENRE=`${CDINFO} -d ${CDDEV} -g`
${CDINFO} -d ${CDDEV} -T  | tr ' ' '_' | tr '&' '_' | tr ',' '_' | tr '\047' '_' > ${LIST}

echo "done."
echo

# Test value
if [ $? -eq 1 -a $TAGONLY -eq 0 ]; then
    echo Disc not present, aborting.
    exit 1
elif [ "$ARTIST" =  "" ]; then
    echo "No information available, please enter manually!"
    echo
    echo -n "Artist ? "
    read ARTIST
    echo -n "Title ? "
    read TITLE
    echo -n "Genre ? "
    read GENRE
    echo 
    echo "If a file with track list is available please enter its name."
    echo "This file should countain one track name by line."
    while [ 1 ]; do
	echo -n "File name ? "
	read L
	if [ -r $L ]; then 
	    cat $L | tr ' ' '_' | tr '&' '_' | tr ',' '_' | tr '\047' '_' > ${LIST}
	    break 
	else 
	    echo "File not found!"
	fi
    done

fi


echo "Artist is \"${ARTIST}\", title is \"${TITLE}\""
echo -n "Genre is  \"${GENRE}\""

my_confirm ", modify"
if [ $? -eq 1 ]; then
    echo -n "Genre ? "
    read GENRE
    echo "New genre is  \"${GENRE}\""
fi

echo
echo "====================================================================="
echo "List of tracks: "
echo
cat ${LIST}
echo "====================================================================="

my_confirm "Confirm ripping/tagging" 
if [ $? -eq 0 ]; then
    echo "Not confirmed, aborting."
    if [ "$L" = "" ]; then
	rm -f ${LIST}
    fi
    exit 1
fi

# Start ripping
if [ $TAGONLY -eq 0 ]; then
    # Set target dir with artist + title
    TARGETDIR=`echo "${ARTIST}-${TITLE}" | tr ' ' '_' | tr '&' '_' | tr ',' '_' | tr '\047' '_' | tr '(' '_' | tr ')' '_' | tr '/' '_'` 
    echo "Using \"${TARGETDIR}\" as target directory"

    if [ ! -r "${TARGETDIR}" ]; then
	mkdir -p "${TARGETDIR}"
	if [ $? -ne 0 ]; then
	    echo "Using \".\" as target directory!"
	    TARGETDIR=.
	fi
    fi

    echo
    echo "Ripping files..."
    echo

    i=1
    cat ${LIST} | while read LINE; do
	cdparanoia -d ${CDDEV} -B "$i" -q - | gogo stdin ${TARGETDIR}/${LINE}.mp3
	i=`expr $i + 1`
    done


    echo 
    echo "Ripping is done."
    echo
else
    TARGETDIR=.
fi

# Tag files
echo
echo "Tagging files..."
echo

i=1
for FILE in ${TARGETDIR}/*.mp3
do
  # Get current title
  T="`head -$i $LIST | tail -1`"
  echo -n "Tagging \"${FILE}\"..."
  mp3info -a "${ARTIST}" -l "${TITLE}" -t "`echo ${T} | tr '_' ' '`" -g "${GENRE}" -n $i  ${FILE}
  echo "done."
  i=`expr $i + 1`
done

if [ "$L" = "" ]; then
    rm -f ${LIST}
fi

echo
echo "Tagging is done."
echo

# iPod transfert?
mount | grep $IPODDIR 1>/dev/null 2>&1
if [ $? -eq 1 ]; then
    echo "Not iPod connected, exiting."
    exit 0
else
    echo "iPod connected, sleeping 30s..."
    sleep 30	
    echo "Sending files to iPod"

    cd ${TARGETDIR}
    gnupod_addsong.pl -m $IPODDIR *.mp3
    mktunes.pl -m $IPODDIR
fi
